package com.example.mapstest


import android.content.Intent
import android.content.pm.PackageManager
import android.location.Address
import android.location.Geocoder
import android.location.Location
import androidx.appcompat.app.AppCompatActivity
import android.os.Bundle
import android.os.Looper
import android.widget.Button
import android.widget.TextView
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.google.android.gms.location.*
import java.util.*

class MainActivity : AppCompatActivity() {
    lateinit var tv: TextView
    lateinit var fusedLocationProviderClient: FusedLocationProviderClient
    var lastLocation: Location? = null

    lateinit var locationCallback: LocationCallback

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        tv = findViewById(R.id.tv)
        val bt = findViewById<Button>(R.id.bt)
        bt.setOnClickListener {
            getLocation()
        }

        val sMap = findViewById<Button>(R.id.bt2)
        sMap.setOnClickListener {
            showMap()
        }

        fusedLocationProviderClient = LocationServices.getFusedLocationProviderClient(this)

        locationCallback = object: LocationCallback() {
            override fun onLocationResult(p0: LocationResult) {
                super.onLocationResult(p0)
                lastLocation = p0.lastLocation
                displayLastLocation()
            }
        }
    }


    fun displayLastLocation() {
        if (lastLocation != null) {
            var address: Address? = null

            var geocoder: Geocoder = Geocoder(this, Locale.getDefault())
            var addresses: List<Address> =
                geocoder.getFromLocation(lastLocation!!.latitude, lastLocation!!.longitude, 1) as List<Address>

            if (addresses != null && addresses.size > 0) {
                address = addresses[0]
            }

            tv.setText("Latitude: " + lastLocation!!.latitude + ", Longitude: " + lastLocation!!.longitude)
            if (address != null)
                tv.append(address.toString())
        }
        else
            tv.setText("Location is not available")
    }


    fun getLocation() {
        if (ContextCompat.checkSelfPermission(this, android.Manifest.permission.ACCESS_FINE_LOCATION)  != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this, arrayOf(android.Manifest.permission.ACCESS_FINE_LOCATION), 1)
        }
        else {
            var locationRequest = LocationRequest.Builder( Priority.PRIORITY_HIGH_ACCURACY, 10000L).build()
            fusedLocationProviderClient.requestLocationUpdates(locationRequest, locationCallback, Looper.getMainLooper())
            getLastLocation()
        }
    }


    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>,
        grantResults: IntArray
    ) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults)
        if (grantResults.size > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED)
            getLastLocation()
    }

    fun getLastLocation() {
        fusedLocationProviderClient.lastLocation.addOnSuccessListener { location ->
            displayLastLocation()
        }
    }

    fun showMap() {
        var i = Intent(this, MapsActivity::class.java)
        i.putExtra("latitude", lastLocation?.latitude)
        i.putExtra("longitude", lastLocation?.longitude)
        startActivity(i)
    }
}